@@ -9,6 +9,7 @@
 #' @param n.adapt Number of iteration to stabilize likelihood
 #' @param thin Interval for thinning likelihoods
 #' @param trace Or FALSE or period to show progress
+#' @param traceML TRUE or FALSE to show ML
 #' @param intermediate Or NULL of period to save intermediate result
 #' @param filename Name of file in which intermediate results are saved
 #' @param adaptive Should an adaptive process for SDProp be used
@@ -46,8 +47,25 @@
 #' Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(0.35, 0.2), 
 #' Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
 #' row.names=c('mean', 'sd'))
+#' # Use of trace and traceML parameters
+#' # trace=1 : Only one likelihood is printed
 #' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
 #' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
+#' # trace=10 : 10 likelihoods are printed
+#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
+#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=10)
+#' # trace=TRUE : all likelihoods are printed
+#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
+#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=TRUE)
+#' # trace=FALSE : No likelihood is printed
+#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
+#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=FALSE)
+#' # traceML=TRUE : values when likelihood is better are shown
+#' mcmc_run <- MHalgoGen(n.iter=100, parameters=parameters_mcmc, data=val, 
+#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=TRUE, traceML=TRUE)
+#' mcmc_run <- MHalgoGen(n.iter=100, parameters=parameters_mcmc, data=val, 
+#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=FALSE, traceML=TRUE)
+#' 
 #' plot(mcmc_run, xlim=c(0, 20))
 #' plot(mcmc_run, xlim=c(0, 10), parameters="sd")
 #' library(graphics)
@@ -123,20 +141,23 @@
 MHalgoGen<-function(likelihood=stop("A likelihood function must be supplied"), 
                     parameters_name="x",
                     parameters=stop("Priors  must be supplied"), ..., 
-                    n.iter=10000, n.chains = 1, n.adapt = 100, thin=30, trace=FALSE, 
+                    n.iter=10000, n.chains = 1, n.adapt = 100, thin=30, 
+                    trace=FALSE, traceML=FALSE, 
                     adaptive = FALSE, adaptive.lag = 500, 
                     adaptive.fun = function(x) {ifelse(x>0.234, 1.3, 0.7)},
                     intermediate=NULL, filename="intermediate.Rdata",
                     previous=NULL)
-
+  
 {
   
   if (!requireNamespace("coda", quietly = TRUE)) {
-     stop("coda package is necessary for this function")
+    stop("coda package is necessary for this function")
   }
   
-# likelihood=NULL; parameters_name="x"; parameters=NULL; n.iter=10000; n.chains = 1; n.adapt = 100; thin=30; trace=FALSE; intermediate=NULL; filename="intermediate.Rdata"; previous=NULL
-# datax <- list(temperatures=result$data, derivate=result$derivate, test=result$test, M0=result$M0, fixed.parameters=result$fixed.parameters, weight=result$weight, out="Likelihood",  progress=FALSE, warnings=FALSE, likelihood=getFromNamespace("info.nests", ns = "embryogrowth"))
+  previousML <- -Inf
+  
+  # likelihood=NULL; parameters_name="x"; parameters=NULL; n.iter=10000; n.chains = 1; n.adapt = 100; thin=30; trace=FALSE; traceML=FALSE; intermediate=NULL; filename="intermediate.Rdata"; previous=NULL
+  # datax <- list(temperatures=result$data, derivate=result$derivate, test=result$test, M0=result$M0, fixed.parameters=result$fixed.parameters, weight=result$weight, out="Likelihood",  progress=FALSE, warnings=FALSE, likelihood=getFromNamespace("info.nests", ns = "embryogrowth"))
   
   if (is.null(previous)) {
     nbvar <- dim(parameters)[1]
@@ -166,6 +187,7 @@ MHalgoGen<-function(likelihood=stop("A likelihood function must be supplied"),
     n.adapt <- previous$n.adapt
     thin <- previous$thin
     trace <- previous$trace
+    traceML <- previous$traceML
     intermediate <- previous$intermediate
     filename <- previous$filename
     deb_kk <- previous$chain
@@ -192,176 +214,199 @@ MHalgoGen<-function(likelihood=stop("A likelihood function must be supplied"),
     pt <- floor((n.adapt+n.iter)/trace)
   }
   
-
-for (kk in deb_kk:n.chains) {
-
-varp <- deb_varp
-varp2 <- deb_varp2
-
-if (is.null(previous)) {
-
-deb_varp2 <- matrix(rep(NA, (nbvar+1)*(n.adapt+n.iter+2)), ncol=nbvar+1)
-deb_varp <- matrix(rep(NA, (nbvar+1)*(n.adapt+n.iter+2)), ncol=nbvar+1)
-colnames(deb_varp)<-c(rownames(parameters), "Ln L")
-colnames(deb_varp2)<-c(rownames(parameters), "Ln L")
-
-
-varp[1, 1:nbvar] <- as.numeric(parameters[1:nbvar, 'Init'])
-
-param <- list(varp[1, 1:nbvar])
-names(param) <- parameters_name
-
-varp[1, "Ln L"] <- -do.call(likelihood, modifyList(datax, param))
-cpt<-1
-varp2[cpt, 1:nbvar] <- varp[1, 1:nbvar]
-varp2[cpt, "Ln L"] <- varp[1, "Ln L"]
-cpt <- 2
-
-
-if (trace) {
-	cat(paste("Chain ", kk, ": [", 1, "] ",as.numeric(varp[1, nbvar+1]), "\n", sep=""))
-} else {
-	cat(paste("Chain ", kk, "\n", sep=""))
-}
-
-MaxL <- varp[1, ]
-
-# sdg=NULL
-
-
-# 18/1/2013
-# for(i in 1:nbvar) sdg<-c(sdg, as.numeric(parameters[i, 'SDProp']))
-# sdg <- c(sdg, as.numeric(parameters[1:nbvar, 'SDProp']))
-sdg <- as.numeric(parameters[1:nbvar, 'SDProp'])
-
-
-# previous <- NULL
-}
-
-
-if (is.data.frame(parameters)) {
-  Prior <- as.matrix(parameters[,c("Prior1", "Prior2")])
-  Limites <- as.matrix(parameters[,c("Min", "Max")])
-} else {
-  Prior<-matrix(as.numeric(parameters[,c("Prior1", "Prior2")]), ncol=2)
-  Limites<-matrix(as.numeric(parameters[,c("Min", "Max")]), ncol=2)
-}
-
-dfun <- as.character(parameters[,"Density"])
-
-# Itérations
-for (i in deb_i:(n.adapt+n.iter)) {
   
-  # est-ce que je sauve où j'en suis
-  if (!is.null(intermediate))
-    if (i %% intermediate==0) {
-      itr <- list(chain=kk, iter=i, varp2=varp2, res=res, 
-                  n.iter=n.iter,
-                  parameters=parameters,
-                  datax=datax,
-                  likelihood=likelihood,
-                  parameters_name=parameters_name,
-                  n.chains=n.chains,
-                  n.adapt=n.adapt,
-                  thin=thin,
-                  trace=trace,
-                  intermediate=intermediate,
-                  filename=filename, 
-                  varp=varp, 
-                  cpt=cpt,
-                  cpt_trace=cpt_trace, 
-                  resL=resL, 
-                  sdg=sdg, MaxL=MaxL, 
-                  adaptive=adaptive,
-                  adaptive.lag=adaptive.lag,
-                  adaptive.fun=adaptive.fun)
-      save(itr, file=filename)
+  for (kk in deb_kk:n.chains) {
+    
+    varp <- deb_varp
+    varp2 <- deb_varp2
+    
+    if (is.null(previous)) {
+      
+      deb_varp2 <- matrix(rep(NA, (nbvar+1)*(n.adapt+n.iter+2)), ncol=nbvar+1)
+      deb_varp <- matrix(rep(NA, (nbvar+1)*(n.adapt+n.iter+2)), ncol=nbvar+1)
+      colnames(deb_varp)<-c(rownames(parameters), "Ln L")
+      colnames(deb_varp2)<-c(rownames(parameters), "Ln L")
+      
+      
+      varp[1, 1:nbvar] <- as.numeric(parameters[1:nbvar, 'Init'])
+      
+      param <- list(varp[1, 1:nbvar])
+      names(param) <- parameters_name
+      
+      # ll <- c(datax, param)
+      # names(ll) <- c("data", parameters_name)
+      
+      varp[1, "Ln L"] <- -do.call(likelihood, modifyList(datax, param))
+      cpt<-1
+      varp2[cpt, 1:nbvar] <- varp[1, 1:nbvar]
+      varp2[cpt, "Ln L"] <- varp[1, "Ln L"]
+      cpt <- 2
+      
+      
+      if (trace) {
+        cat(paste("Chain ", kk, ": [", 1, "] ",as.numeric(varp[1, nbvar+1]), "\n", sep=""))
+      } else {
+        cat(paste("Chain ", kk, "\n", sep=""))
+      }
+      
+      if (traceML) {
+        if (as.numeric(varp[1, "Ln L"]) > previousML) {
+          previousML <- varp[1, "Ln L"]
+          cat(d(varp[1, 1:nbvar]))
+          cat("\n")
+        }
+      }
+      MaxL <- varp[1, ]
+      
+      # sdg=NULL
+      
+      
+      # 18/1/2013
+      # for(i in 1:nbvar) sdg<-c(sdg, as.numeric(parameters[i, 'SDProp']))
+      # sdg <- c(sdg, as.numeric(parameters[1:nbvar, 'SDProp']))
+      sdg <- as.numeric(parameters[1:nbvar, 'SDProp'])
+      
+      
+      # previous <- NULL
     }
-  
-  
-  ###### Pas clair si previous
-    deb_i <- 2
-    Lprevious <- LpreviousT <- varp[i-1, "Ln L"]
-	newvarp <- varp[i-1, 1:nbvar]
-	
-	for (j in 1:nbvar) {	
-		# Nouveau paramètre
-		propvarp <- newvarp
-		propvarp[j] <- propvarp[j]+rnorm(1, mean=0, sd=sdg[j])
-
-		if (propvarp[j]<=Limites[j,2] && propvarp[j]>=Limites[j,1]) 
-			{
-		  param <- list(propvarp)
-		  names(param) <- parameters_name
-		  Lprevious2 <- -do.call(likelihood, modifyList(datax, param))
-			logratio <- get(dfun[j])(propvarp[j],Prior[j,1],Prior[j,2],log=TRUE) + Lprevious2 -
-					(get(dfun[j])(newvarp[j],Prior[j,1],Prior[j,2],log=TRUE)+LpreviousT)
-			alpha<-min(c(1,exp(logratio)))
-			# 15/2/2015 Pour éviter des erreurs
-			if (!is.finite(alpha)) alpha <- -1
-			if (runif(1, min=0, max=1)<=alpha) {
-			  newvarp <- propvarp
-			  LpreviousT <- Lprevious2
-			  } 
-			}
-	}		
-	varp[i, 1:nbvar] <- newvarp
-	varp[i, "Ln L"] <- LpreviousT
-
-	if (MaxL["Ln L"] < varp[i, "Ln L"]) { MaxL <- varp[i,]}
-  
-
-# 6/10/2012: Je stocke tout	
-#	if ((i>n.adapt) && ((i%%thin)==0)) {
-		varp2[cpt, 1:nbvar]<-newvarp
-		varp2[cpt, "Ln L"]<-varp[i, "Ln L"]
-		cpt<-cpt+1
-#	}
-	if (tf) {
-    cpt_trace <- cpt_trace+1
-    if (cpt_trace>=pt) {
-	    cat(paste("Chain ", kk, ": [", i, "] ", as.numeric(varp[i, "Ln L"]), "\n", sep=""))
-      cpt_trace <- 0
+    
+    
+    if (is.data.frame(parameters)) {
+      Prior <- as.matrix(parameters[,c("Prior1", "Prior2")])
+      Limites <- as.matrix(parameters[,c("Min", "Max")])
+    } else {
+      Prior<-matrix(as.numeric(parameters[,c("Prior1", "Prior2")]), ncol=2)
+      Limites<-matrix(as.numeric(parameters[,c("Min", "Max")]), ncol=2)
     }
-	}
-		
-		
-		if (adaptive) {
-		  if ((i %% adaptive.lag) == 0) {
-
-#		    sdp <- 
-		    for (j in 1:nbvar) {
-		      dta <- varp[(i-adaptive.lag):i, j]
-		      txaccept <- sum(diff(dta)!=0)/(length(dta)-1)
-		      sdg[j] <- adaptive.fun(txaccept)*sdg[j]
-		    }
-		    # print(sdg)
-		  }
-		}
-		
-}
-
-lp <- getFromNamespace("as.mcmc", ns="coda")(varp2[(n.adapt+1):(cpt-1), 1:nbvar])
-lp <- getFromNamespace("mcmc", ns="coda")(data=lp, start=n.adapt+1, end=n.iter+n.adapt, thin=thin)
-
-# Ca c'est uniquement pour les chaînes
-# Ce n'est pas ca qui fait perdre du temps
-
-res<-c(res, list(lp))
-resL <- c(resL, list(varp2[(n.adapt+1):(cpt-1), "Ln L"]))
-}
-
-names(res) <- 1:n.chains
-res <- getFromNamespace("as.mcmc.list", ns="coda")(res)
-
-cat("Best likelihood for: \n")
-for (j in 1:nbvar) {cat(paste(names(MaxL[j]), "=", MaxL[j], "\n"))}
-
-out <- (list(resultMCMC=res, resultLnL=resL, 
-             parametersMCMC=list(parameters=parameters, n.iter=n.iter, n.chains=n.chains, n.adapt=n.adapt, thin=thin, 
-                                 SDProp.end=structure(sdg, .Names=rownames(parameters)), 
-                                 control=datax)))
-class(out) <- "mcmcComposite"
-return(out)
-
+    
+    dfun <- as.character(parameters[,"Density"])
+    
+    # Itérations
+    for (i in deb_i:(n.adapt+n.iter)) {
+      
+      # est-ce que je sauve où j'en suis
+      if (!is.null(intermediate))
+        if (i %% intermediate==0) {
+          itr <- list(chain=kk, iter=i, varp2=varp2, res=res, 
+                      n.iter=n.iter,
+                      parameters=parameters,
+                      datax=datax,
+                      likelihood=likelihood,
+                      parameters_name=parameters_name,
+                      n.chains=n.chains,
+                      n.adapt=n.adapt,
+                      thin=thin,
+                      trace=trace,
+                      traceML=traceML,
+                      intermediate=intermediate,
+                      filename=filename, 
+                      varp=varp, 
+                      cpt=cpt,
+                      cpt_trace=cpt_trace, 
+                      resL=resL, 
+                      sdg=sdg, MaxL=MaxL, 
+                      adaptive=adaptive,
+                      adaptive.lag=adaptive.lag,
+                      adaptive.fun=adaptive.fun)
+          save(itr, file=filename)
+        }
+      
+      
+      ###### Pas clair si previous
+      deb_i <- 2
+      Lprevious <- LpreviousT <- varp[i-1, "Ln L"]
+      newvarp <- varp[i-1, 1:nbvar]
+      
+      for (j in 1:nbvar) {	
+        # Nouveau paramètre
+        propvarp <- newvarp
+        propvarp[j] <- propvarp[j]+rnorm(1, mean=0, sd=sdg[j])
+        
+        if (propvarp[j]<=Limites[j,2] && propvarp[j]>=Limites[j,1]) 
+        {
+          param <- list(propvarp)
+          names(param) <- parameters_name
+          Lprevious2 <- -do.call(likelihood, modifyList(datax, param))
+          logratio <- get(dfun[j])(propvarp[j],Prior[j,1],Prior[j,2],log=TRUE) + Lprevious2 -
+            (get(dfun[j])(newvarp[j],Prior[j,1],Prior[j,2],log=TRUE)+LpreviousT)
+          alpha<-min(c(1,exp(logratio)))
+          # 15/2/2015 Pour éviter des erreurs
+          if (!is.finite(alpha)) alpha <- -1
+          if (runif(1, min=0, max=1)<=alpha) {
+            newvarp <- propvarp
+            LpreviousT <- Lprevious2
+          } 
+        }
+      }		
+      varp[i, 1:nbvar] <- newvarp
+      varp[i, "Ln L"] <- LpreviousT
+      
+      if (MaxL["Ln L"] < varp[i, "Ln L"]) {
+        MaxL <- varp[i, ]
+      }
+      
+      
+      # 6/10/2012: Je stocke tout	
+      #	if ((i>n.adapt) && ((i%%thin)==0)) {
+      varp2[cpt, 1:nbvar] <- newvarp
+      varp2[cpt, "Ln L"] <- varp[i, "Ln L"]
+      cpt <- cpt + 1
+      #	}
+      
+      if (tf) {
+        cpt_trace <- cpt_trace+1
+        if (cpt_trace>=pt) {
+          cat(paste("Chain ", kk, ": [", i, "] ", as.numeric(varp[i, "Ln L"]), "\n", sep=""))
+          cpt_trace <- 0
+        }
+      }
+      
+      if (traceML) {
+        if (as.numeric(varp[i, "Ln L"]) > previousML) {
+          previousML <- varp[i, "Ln L"]
+          cat(d(varp[i, 1:nbvar]))
+          cat("\n")
+        }
+      }
+      
+      
+      
+      if (adaptive) {
+        if ((i %% adaptive.lag) == 0) {
+          
+          #		    sdp <- 
+          for (j in 1:nbvar) {
+            dta <- varp[(i-adaptive.lag):i, j]
+            txaccept <- sum(diff(dta)!=0)/(length(dta)-1)
+            sdg[j] <- adaptive.fun(txaccept)*sdg[j]
+          }
+          # print(sdg)
+        }
+      }
+      
+    }
+    
+    lp <- getFromNamespace("as.mcmc", ns="coda")(varp2[(n.adapt+1):(cpt-1), 1:nbvar])
+    lp <- getFromNamespace("mcmc", ns="coda")(data=lp, start=n.adapt+1, end=n.iter+n.adapt, thin=thin)
+    
+    # Ca c'est uniquement pour les chaînes
+    # Ce n'est pas ca qui fait perdre du temps
+    
+    res<-c(res, list(lp))
+    resL <- c(resL, list(varp2[(n.adapt+1):(cpt-1), "Ln L"]))
+  }
+  
+  names(res) <- 1:n.chains
+  res <- getFromNamespace("as.mcmc.list", ns="coda")(res)
+  
+  cat("Best likelihood for: \n")
+  for (j in 1:nbvar) {cat(paste(names(MaxL[j]), "=", MaxL[j], "\n"))}
+  
+  out <- (list(resultMCMC=res, resultLnL=resL, 
+               parametersMCMC=list(parameters=parameters, n.iter=n.iter, n.chains=n.chains, n.adapt=n.adapt, thin=thin, 
+                                   SDProp.end=structure(sdg, .Names=rownames(parameters)), 
+                                   control=datax)))
+  class(out) <- "mcmcComposite"
+  return(out)
+  
 }
